@@ -26,6 +26,12 @@
 #include <openssl/hmac.h>
 #include <openssl/rsa.h>
 #include <openssl/sha.h>
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+# include <openssl/core_names.h>
+# include <openssl/param_build.h>
+#else
+# include <openssl/rsa.h>
+#endif
 
 #include "swtpm.h"
 #include "swtpm_utils.h"
@@ -50,26 +56,34 @@ struct tpm_resp_header {
 
 static int swtpm_start(struct swtpm *self)
 {
-    g_autofree gchar *tpmstate_dir = g_strdup_printf("dir=%s", self->state_path);
-    g_autofree gchar *pidfile_file = NULL;
+    g_autofree gchar *tpmstate = g_strdup_printf("backend-uri=%s", self->state_path);
+    g_autofree gchar *pidfile_arg = NULL;
     g_autofree gchar *server_fd = NULL;
     g_autofree gchar *ctrl_fd = NULL;
     g_autofree gchar *keyopts = NULL;
+    g_autofree gchar *logop = NULL;
     g_autofree gchar **argv = NULL;
     struct stat statbuf;
     gboolean success;
     GError *error = NULL;
     unsigned ctr;
+    int pidfile_fd;
+    int ret = 1;
+    char pidfile[] = "/tmp/.swtpm_setup.pidfile.XXXXXX";
 
-    self->pidfile = g_strjoin(G_DIR_SEPARATOR_S,
-                              self->state_path, ".swtpm_setup.pidfile", NULL);
-    pidfile_file = g_strdup_printf("file=%s", self->pidfile);
+    pidfile_fd = mkstemp(pidfile);
+    if (pidfile_fd < 0) {
+        logerr(self->logfile, "Could not create pidfile: %s\n", strerror(errno));
+        goto error_no_pidfile;
+    }
+    // pass filename rather than fd (Cygwin)
+    pidfile_arg = g_strdup_printf("file=%s", pidfile);
 
     argv = concat_arrays(self->swtpm_exec_l,
                          (gchar*[]){
                               "--flags", "not-need-init,startup-clear",
-                              "--tpmstate", tpmstate_dir,
-                              "--pid", pidfile_file,
+                              "--tpmstate", tpmstate,
+                              "--pid", pidfile_arg,
 #if 0
                               "--log", "file=/tmp/log,level=20",
 #endif
@@ -84,15 +98,20 @@ static int swtpm_start(struct swtpm *self)
         argv = concat_arrays(argv, (gchar*[]){"--key", keyopts, NULL}, TRUE);
     }
 
+    if (gl_LOGFILE != NULL) {
+        logop = g_strdup_printf("file=%s", gl_LOGFILE);
+        argv = concat_arrays(argv, (gchar*[]){"--log", logop, NULL}, TRUE);
+    }
+
     if (socketpair(AF_UNIX, SOCK_STREAM, 0, self->ctrl_fds) != 0) {
         logerr(self->logfile, "Could not create socketpair: %s\n", strerror(errno));
-        return 1;
+        goto error;
     }
     ctrl_fd = g_strdup_printf("type=unixio,clientfd=%d", self->ctrl_fds[1]);
 
     if (socketpair(AF_UNIX, SOCK_STREAM, 0, self->data_fds) != 0) {
         logerr(self->logfile, "Could not create socketpair: %s\n", strerror(errno));
-        return 1;
+        goto error;
     }
     server_fd = g_strdup_printf("type=tcp,fd=%d", self->data_fds[1]);
 
@@ -102,11 +121,6 @@ static int swtpm_start(struct swtpm *self)
                              NULL
                          }, TRUE);
 
-    if (self->cops->remove_pidfile(self) != 0) {
-        logerr(self->logfile, "Could not remove PID file\n");
-        return 1;
-    }
-
 #if 0
     {
         g_autofree gchar *join = g_strjoinv(" ", argv);
@@ -120,34 +134,32 @@ static int swtpm_start(struct swtpm *self)
     if (!success) {
         logerr(self->logfile, "Could not start swtpm: %s\n", error->message);
         g_error_free(error);
-        return 1;
+        goto error;
     }
 
-    /* wait until the pidfile appears or swtpm terminates */
+    /* wait until the pidfile is written to or swtpm terminates */
     for (ctr = 0; ctr < 1000; ctr++) {
         if (kill(self->pid, 0) < 0) {
             /* swtpm terminated */
             self->pid = 0;
             logerr(self->logfile, "swtpm process terminated unexpectedly.\n");
             self->cops->stop(self);
-            return 1;
+            goto error;
         }
-        if (stat(self->pidfile, &statbuf) == 0) {
+        if (fstat(pidfile_fd, &statbuf) == 0 && statbuf.st_size > 0) {
             printf("TPM is listening on Unix socket.\n");
-            return 0;
+            ret = 0;
+            break;
         }
         usleep(5000);
     }
 
-    return 1;
-}
+error:
+    close(pidfile_fd);
+    unlink(pidfile);
 
-/* Remove the PID file; return an error if file could not be removed */
-static int swtpm_remove_pidfile(struct swtpm *self)
-{
-    if (self->pidfile && unlink(self->pidfile) < 0 && errno != ENOENT)
-        return 1;
-    return 0;
+error_no_pidfile:
+    return ret;
 }
 
 /* Stop a running swtpm instance and close all the file descriptors connecting to it */
@@ -171,7 +183,6 @@ static void swtpm_stop(struct swtpm *self)
 
         self->pid = 0;
     }
-    self->cops->remove_pidfile(self);
 
     if (self->ctrl_fds[0] >= 0) {
         close(self->ctrl_fds[0]);
@@ -303,7 +314,6 @@ static const struct swtpm_cops swtpm_cops = {
     .start = swtpm_start,
     .stop = swtpm_stop,
     .destroy = swtpm_destroy,
-    .remove_pidfile = swtpm_remove_pidfile,
     .ctrl_shutdown = swtpm_ctrl_shutdown,
     .ctrl_get_tpm_specs_and_attrs = swtpm_ctrl_get_tpm_specs_and_attrs,
 };
@@ -425,7 +435,7 @@ static int swtpm_tpm2_createprimary_rsa(struct swtpm *self, uint32_t primaryhand
                                         unsigned int rsa_keysize, gboolean havenonce, size_t off,
                                         uint32_t *curr_handle,
                                         unsigned char *ektemplate, size_t *ektemplate_len,
-                                        gchar **ekparam);
+                                        gchar **ekparam, const gchar **key_description);
 
 static int swtpm_tpm2_write_nvram(struct swtpm *self, uint32_t nvindex, uint32_t nvindexattrs,
                                   const unsigned char *data, size_t data_len, gboolean lock_nvram,
@@ -666,7 +676,7 @@ static int swtpm_tpm2_createprimary_ek_rsa(struct swtpm *self, unsigned int rsa_
                                            gboolean allowsigning, gboolean decryption,
                                            uint32_t *curr_handle,
                                            unsigned char *ektemplate, size_t *ektemplate_len,
-                                           gchar **ekparam)
+                                           gchar **ekparam, const gchar **key_description)
 {
     unsigned char authpolicy[48];
     size_t authpolicy_len;
@@ -679,23 +689,23 @@ static int swtpm_tpm2_createprimary_ek_rsa(struct swtpm *self, unsigned int rsa_
 
     if (rsa_keysize == 2048) {
         authpolicy_len = 32;
-        memcpy(authpolicy, (unsigned char []){
+        memcpy(authpolicy, ((unsigned char []){
             0x83, 0x71, 0x97, 0x67, 0x44, 0x84, 0xb3, 0xf8, 0x1a, 0x90, 0xcc, 0x8d,
             0x46, 0xa5, 0xd7, 0x24, 0xfd, 0x52, 0xd7, 0x6e, 0x06, 0x52, 0x0b, 0x64,
             0xf2, 0xa1, 0xda, 0x1b, 0x33, 0x14, 0x69, 0xaa
-        }, authpolicy_len);
+        }), authpolicy_len);
         keyflags = 0;
         symkeylen = 128;
         havenonce = TRUE;
         addlen = 0;
     } else if (rsa_keysize == 3072) {
         authpolicy_len = 48;
-        memcpy(authpolicy, (unsigned char []){
+        memcpy(authpolicy, ((unsigned char []){
             0xB2, 0x6E, 0x7D, 0x28, 0xD1, 0x1A, 0x50, 0xBC, 0x53, 0xD8, 0x82, 0xBC,
             0xF5, 0xFD, 0x3A, 0x1A, 0x07, 0x41, 0x48, 0xBB, 0x35, 0xD3, 0xB4, 0xE4,
             0xCB, 0x1C, 0x0A, 0xD9, 0xBD, 0xE4, 0x19, 0xCA, 0xCB, 0x47, 0xBA, 0x09,
             0x69, 0x96, 0x46, 0x15, 0x0F, 0x9F, 0xC0, 0x00, 0xF3, 0xF8, 0x0E, 0x12
-        }, authpolicy_len);
+        }), authpolicy_len);
         keyflags = 0x40;
         symkeylen = 256;
         havenonce = FALSE;
@@ -712,7 +722,7 @@ static int swtpm_tpm2_createprimary_ek_rsa(struct swtpm *self, unsigned int rsa_
         keyflags |= 0x000600b2;
         // symmetric: TPM_ALG_NULL
         symkeydata_len = 2;
-        memcpy(symkeydata, (unsigned char[]) {AS2BE(TPM2_ALG_NULL)}, symkeydata_len);
+        memcpy(symkeydata, ((unsigned char[]) {AS2BE(TPM2_ALG_NULL)}), symkeydata_len);
         off = 72 + addlen;
     } else if (allowsigning) {
         // keyflags: fixedTPM, fixedParent, sensitiveDatOrigin,
@@ -720,7 +730,7 @@ static int swtpm_tpm2_createprimary_ek_rsa(struct swtpm *self, unsigned int rsa_
         keyflags |= 0x000400b2;
         // symmetric: TPM_ALG_NULL
         symkeydata_len = 2;
-        memcpy(symkeydata, (unsigned char[]) {AS2BE(TPM2_ALG_NULL)}, symkeydata_len);
+        memcpy(symkeydata, ((unsigned char[]) {AS2BE(TPM2_ALG_NULL)}), symkeydata_len);
         off = 72 + addlen;
     } else {
         // keyflags: fixedTPM, fixedParent, sensitiveDatOrigin,
@@ -729,7 +739,7 @@ static int swtpm_tpm2_createprimary_ek_rsa(struct swtpm *self, unsigned int rsa_
         // symmetric: TPM_ALG_AES, 128bit or 256bit, TPM_ALG_CFB
         symkeydata_len = 6;
         memcpy(symkeydata,
-               (unsigned char[]) {AS2BE(TPM2_ALG_AES), AS2BE(symkeylen), AS2BE(TPM2_ALG_CFB)},
+               ((unsigned char[]) {AS2BE(TPM2_ALG_AES), AS2BE(symkeylen), AS2BE(TPM2_ALG_CFB)}),
                symkeydata_len);
         off = 76 + addlen;
     }
@@ -738,7 +748,7 @@ static int swtpm_tpm2_createprimary_ek_rsa(struct swtpm *self, unsigned int rsa_
                                         symkeydata, symkeydata_len,
                                         authpolicy, authpolicy_len, rsa_keysize,
                                         havenonce, off, curr_handle,
-                                        ektemplate, ektemplate_len, ekparam);
+                                        ektemplate, ektemplate_len, ekparam, key_description);
 }
 
 /* Create an RSA key with the given parameters */
@@ -748,7 +758,7 @@ static int swtpm_tpm2_createprimary_rsa(struct swtpm *self, uint32_t primaryhand
                                         unsigned int rsa_keysize, gboolean havenonce, size_t off,
                                         uint32_t *curr_handle,
                                         unsigned char *ektemplate, size_t *ektemplate_len,
-                                        gchar **ekparam)
+                                        gchar **ekparam, const gchar **key_description)
 {
     const unsigned char *nonce;
     size_t nonce_len;
@@ -768,6 +778,8 @@ static int swtpm_tpm2_createprimary_rsa(struct swtpm *self, uint32_t primaryhand
         nonce = NONCE_RSA2048;
         nonce_len = sizeof(NONCE_RSA2048);
         hashalg = TPM2_ALG_SHA256;
+        if (key_description)
+            *key_description = "rsa2048";
     } else if (rsa_keysize == 3072) {
         if (!havenonce) {
            nonce = NONCE_EMPTY;
@@ -777,6 +789,8 @@ static int swtpm_tpm2_createprimary_rsa(struct swtpm *self, uint32_t primaryhand
            nonce_len = sizeof(NONCE_RSA3072);
         }
         hashalg = TPM2_ALG_SHA384;
+        if (key_description)
+            *key_description = "rsa3072";
     } else {
         logerr(self->logfile, "Internal error in %s: unsupported RSA keysize %d.\n",
                __func__, rsa_keysize);
@@ -867,7 +881,7 @@ static int swtpm_tpm2_createprimary_ecc(struct swtpm *self, uint32_t primaryhand
                                         const unsigned char *nonce, size_t nonce_len,
                                         size_t off, uint32_t *curr_handle,
                                         unsigned char *ektemplate, size_t *ektemplate_len,
-                                        gchar **ekparam)
+                                        gchar **ekparam, const gchar **key_description)
 {
     struct tpm_req_header hdr = TPM_REQ_HEADER_INITIALIZER(TPM2_ST_SESSIONS, 0, TPM2_CC_CREATEPRIMARY);
     struct tpm2_authblock authblock = TPM2_AUTHBLOCK_INITIALIZER(TPM2_RS_PW, 0, 0, 0);
@@ -936,6 +950,8 @@ static int swtpm_tpm2_createprimary_ecc(struct swtpm *self, uint32_t primaryhand
     if (curveid == TPM2_ECC_NIST_P384) {
         exp_ksize = 48;
         cid = "secp384r1";
+        if (key_description)
+            *key_description = cid;
     } else {
         logerr(self->logfile, "Unknown curveid 0x%x\n", curveid);
         return 1;
@@ -988,7 +1004,7 @@ static int swtpm_tpm2_createprimary_spk_ecc_nist_p384(struct swtpm *self,
     return swtpm_tpm2_createprimary_ecc(self, TPM2_RH_OWNER, keyflags, symkeydata, symkeydata_len,
                                         authpolicy, authpolicy_len, TPM2_ECC_NIST_P384, TPM2_ALG_SHA384,
                                         NONCE_ECC_384, sizeof(NONCE_ECC_384), off, curr_handle,
-                                        NULL, 0, NULL);
+                                        NULL, 0, NULL, NULL);
 }
 
 static int swtpm_tpm2_createprimary_spk_rsa(struct swtpm *self, unsigned int rsa_keysize,
@@ -1009,13 +1025,13 @@ static int swtpm_tpm2_createprimary_spk_rsa(struct swtpm *self, unsigned int rsa
 
     symkeydata_len = 6;
     memcpy(symkeydata,
-           (unsigned char[]) {AS2BE(TPM2_ALG_AES), AS2BE(symkeylen), AS2BE(TPM2_ALG_CFB)},
+           ((unsigned char[]) {AS2BE(TPM2_ALG_AES), AS2BE(symkeylen), AS2BE(TPM2_ALG_CFB)}),
            symkeydata_len);
 
     return swtpm_tpm2_createprimary_rsa(self, TPM2_RH_OWNER, keyflags,
                                         symkeydata, symkeydata_len,
                                         authpolicy, authpolicy_len, rsa_keysize, TRUE,
-                                        off, curr_handle, NULL, 0, NULL);
+                                        off, curr_handle, NULL, 0, NULL, NULL);
 }
 
 /* Create either an ECC or RSA storage primary key */
@@ -1044,7 +1060,7 @@ static int swtpm_tpm2_create_spk(struct swtpm *self, gboolean isecc, unsigned in
 static int swtpm_tpm2_createprimary_ek_ecc_nist_p384(struct swtpm *self, gboolean allowsigning,
                                                      gboolean decryption, uint32_t *curr_handle,
                                                      unsigned char *ektemplate, size_t *ektemplate_len,
-                                                     gchar **ekparam)
+                                                     gchar **ekparam, const char **key_description)
 {
     unsigned char authpolicy[48]= {
         0xB2, 0x6E, 0x7D, 0x28, 0xD1, 0x1A, 0x50, 0xBC, 0x53, 0xD8, 0x82, 0xBC,
@@ -1065,7 +1081,7 @@ static int swtpm_tpm2_createprimary_ek_ecc_nist_p384(struct swtpm *self, gboolea
         keyflags = 0x000600f2;
         // symmetric: TPM_ALG_NULL
         symkeydata_len = 2;
-        memcpy(symkeydata, (unsigned char[]){AS2BE(TPM2_ALG_NULL)}, symkeydata_len);
+        memcpy(symkeydata, ((unsigned char[]){AS2BE(TPM2_ALG_NULL)}), symkeydata_len);
         off = 86;
     } else if (allowsigning) {
         // keyflags: fixedTPM, fixedParent, sensitiveDatOrigin,
@@ -1073,7 +1089,7 @@ static int swtpm_tpm2_createprimary_ek_ecc_nist_p384(struct swtpm *self, gboolea
         keyflags = 0x000400f2;
         // symmetric: TPM_ALG_NULL
         symkeydata_len = 2;
-        memcpy(symkeydata, (unsigned char[]){AS2BE(TPM2_ALG_NULL)}, symkeydata_len);
+        memcpy(symkeydata, ((unsigned char[]){AS2BE(TPM2_ALG_NULL)}), symkeydata_len);
         off = 86;
     } else {
         // keyflags: fixedTPM, fixedParent, sensitiveDatOrigin,
@@ -1082,7 +1098,7 @@ static int swtpm_tpm2_createprimary_ek_ecc_nist_p384(struct swtpm *self, gboolea
         // symmetric: TPM_ALG_AES, 256bit, TPM_ALG_CFB
         symkeydata_len = 6;
         memcpy(symkeydata,
-               (unsigned char[]){ AS2BE(TPM2_ALG_AES), AS2BE(256), AS2BE(TPM2_ALG_CFB)},
+               ((unsigned char[]){ AS2BE(TPM2_ALG_AES), AS2BE(256), AS2BE(TPM2_ALG_CFB)}),
                symkeydata_len);
         off = 90;
     }
@@ -1090,7 +1106,7 @@ static int swtpm_tpm2_createprimary_ek_ecc_nist_p384(struct swtpm *self, gboolea
     ret = swtpm_tpm2_createprimary_ecc(self, TPM2_RH_ENDORSEMENT, keyflags, symkeydata, symkeydata_len,
                                        authpolicy, authpolicy_len, TPM2_ECC_NIST_P384, TPM2_ALG_SHA384,
                                        NONCE_EMPTY, sizeof(NONCE_EMPTY), off, curr_handle,
-                                       ektemplate, ektemplate_len, ekparam);
+                                       ektemplate, ektemplate_len, ekparam, key_description);
     if (ret != 0)
        logerr(self->logfile, "%s failed\n", __func__);
 
@@ -1100,7 +1116,7 @@ static int swtpm_tpm2_createprimary_ek_ecc_nist_p384(struct swtpm *self, gboolea
 /* Create an ECC or RSA EK */
 static int swtpm_tpm2_create_ek(struct swtpm *self, gboolean isecc, unsigned int rsa_keysize,
                                 gboolean allowsigning, gboolean decryption, gboolean lock_nvram,
-                                gchar **ekparam)
+                                gchar **ekparam, const  gchar **key_description)
 {
     uint32_t tpm2_ek_handle, nvindex, curr_handle;
     const char *keytype;
@@ -1128,10 +1144,11 @@ static int swtpm_tpm2_create_ek(struct swtpm *self, gboolean isecc, unsigned int
     }
     if (isecc)
         ret = swtpm_tpm2_createprimary_ek_ecc_nist_p384(self, allowsigning, decryption, &curr_handle,
-                                                        ektemplate, &ektemplate_len, ekparam);
+                                                        ektemplate, &ektemplate_len, ekparam,
+                                                        key_description);
     else
         ret = swtpm_tpm2_createprimary_ek_rsa(self, rsa_keysize, allowsigning, decryption, &curr_handle,
-                                              ektemplate, &ektemplate_len, ekparam);
+                                              ektemplate, &ektemplate_len, ekparam, key_description);
 
     if (ret == 0)
         ret = swtpm_tpm2_evictcontrol(self, curr_handle, tpm2_ek_handle);
@@ -1538,7 +1555,9 @@ static int swtpm_tpm12_take_ownership(struct swtpm *self, const unsigned char ow
     EVP_PKEY_CTX *ctx = NULL;
     BIGNUM *exp = BN_new();
     BIGNUM *mod = NULL;
+#if OPENSSL_VERSION_NUMBER < 0x30000000L
     RSA *rsakey = RSA_new();
+#endif
     int ret = 1;
     const EVP_MD *sha1 = EVP_sha1();
     g_autofree unsigned char *enc_owner_auth = g_malloc(pubek_len);
@@ -1573,19 +1592,54 @@ static int swtpm_tpm12_take_ownership(struct swtpm *self, const unsigned char ow
         goto error_free_bn;
     }
 
-    pkey = EVP_PKEY_new();
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+    ctx = EVP_PKEY_CTX_new_from_name(NULL, "rsa", NULL);
+    if (ctx != NULL) {
+        OSSL_PARAM_BLD *bld = OSSL_PARAM_BLD_new();
+        OSSL_PARAM *params;
 
-#if OPENSSL_VERSION_NUMBER < 0x10100000
+        if (bld == NULL ||
+            OSSL_PARAM_BLD_push_BN(bld, OSSL_PKEY_PARAM_RSA_E, exp) != 1 ||
+            OSSL_PARAM_BLD_push_BN(bld, OSSL_PKEY_PARAM_RSA_N, mod) != 1 ||
+            (params = OSSL_PARAM_BLD_to_param(bld)) == NULL) {
+            OSSL_PARAM_BLD_free(bld);
+            goto error_free_bn;
+        }
+        OSSL_PARAM_BLD_free(bld);
+
+        if (EVP_PKEY_fromdata_init(ctx) != 1 ||
+            EVP_PKEY_fromdata(ctx, &pkey, EVP_PKEY_PUBLIC_KEY, params) != 1) {
+            logerr(self->logfile, "Could not set pkey parameters!\n");
+            OSSL_PARAM_free(params);
+            goto error_free_bn;
+        }
+        OSSL_PARAM_free(params);
+
+        EVP_PKEY_CTX_free(ctx);
+    } else {
+        logerr(self->logfile, "Could not create key creation context!\n");
+        goto error_free_bn;
+    }
+    ctx = EVP_PKEY_CTX_new_from_pkey(NULL, pkey, NULL);
+    if (ctx == NULL)
+        goto error_free_bn;
+#else
+    pkey = EVP_PKEY_new();
+    if (pkey == NULL) {
+        logerr(self->logfile, "Could not allocate pkey!\n");
+        goto error_free_bn;
+    }
+
+# if OPENSSL_VERSION_NUMBER < 0x10100000
     rsakey->n = mod;
     rsakey->e = exp;
-#else
+# else
     if (RSA_set0_key(rsakey, mod, exp, NULL) != 1) {
         logerr(self->logfile, "Could not create public RSA key!\n");
         goto error_free_bn;
     }
-#endif
-    if (pkey == NULL ||
-        EVP_PKEY_assign_RSA(pkey, rsakey) != 1) {
+# endif
+    if (EVP_PKEY_assign_RSA(pkey, rsakey) != 1) {
         logerr(self->logfile, "Could not create public RSA key!\n");
         goto error_free_pkey_and_rsa;
     }
@@ -1593,6 +1647,7 @@ static int swtpm_tpm12_take_ownership(struct swtpm *self, const unsigned char ow
     ctx = EVP_PKEY_CTX_new(pkey, NULL);
     if (ctx == NULL)
         goto error_free_pkey;
+#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
 
     if (EVP_PKEY_encrypt_init(ctx) < 1 ||
         EVP_PKEY_CTX_set_rsa_padding(ctx, RSA_PKCS1_OAEP_PADDING) < 1 ||
@@ -1702,15 +1757,23 @@ static int swtpm_tpm12_take_ownership(struct swtpm *self, const unsigned char ow
 error:
     EVP_PKEY_free(pkey);
     EVP_PKEY_CTX_free(ctx);
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+    BN_free(exp);
+    BN_free(mod);
+#endif
     return ret;
 
 error_free_bn:
     BN_free(exp);
     BN_free(mod);
 
+#if OPENSSL_VERSION_NUMBER < 0x30000000L
 error_free_pkey_and_rsa:
     RSA_free(rsakey);
 error_free_pkey:
+#else
+    EVP_PKEY_CTX_free(ctx);
+#endif
     EVP_PKEY_free(pkey);
 
     return 1;
@@ -1887,7 +1950,6 @@ struct swtpm2 *swtpm2_new(gchar **swtpm_exec_l, const gchar *state_path,
 void swtpm_free(struct swtpm *swtpm) {
     if (!swtpm)
         return;
-    g_free(swtpm->pidfile);
     g_free(swtpm);
 }
 
